/* -*- Mode: idl; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */


#include "nsISupports.idl"

interface nsIXULTemplateBuilder;

// An nsIXULBuilderListener object is a listener that will be notified
// when a template builder rebuilds its content.
[scriptable, uuid(ac46be8f-c863-4c23-84a2-d0fcc8dfa9f4)]
interface nsIXULBuilderListener: nsISupports {

  /**
   * Called before a template builder rebuilds its content.
   * @param aBuilder the template builder that rebuilds the content.
   */
  void willRebuild(in nsIXULTemplateBuilder aBuilder);

  /**
   * Called after a template builder has rebuilt its content.
   * @param aBuilder the template builder that has rebuilt the content.
   */
  void didRebuild(in nsIXULTemplateBuilder aBuilder);

};
